unit Main;

interface

uses
  Windows, Messages, ShellApi, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SnapWindow, ComCtrls, ActnList, StdActns, ImgList, Menus, Clipbrd;

const
  TITLE    = 'YAC Log Viewer';

  CALL_SIG = '***** INCOMING CALL:';
  NAME_SIG = 'NAME:';
  NUMB_SIG = 'NUMBER:';

  DELIM    = #9;

  LOADING_LOG = 'Loading call log...';

  WM_USER_ACTIVATE_PREV = WM_USER + $100;

type
  Tfrm_Main = class(TForm)
    snap: TSnapWindow;
    stat: TStatusBar;
    lst_Calls: TListView;
    mnu_Popup: TPopupMenu;
    act_List1: TActionList;
    img_List1: TImageList;
    act_Copy: TEditCopy;
    act_Google: TAction;
    mnu_Copy: TMenuItem;
    mnu_Google: TMenuItem;
    act_CopyNum: TEditCopy;
    mnu_CopyNum: TMenuItem;
    N1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lst_CallsColumnClick(Sender: TObject; Column: TListColumn);
    procedure lst_CallsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure act_GoogleExecute(Sender: TObject);
    procedure act_CopyExecute(Sender: TObject);
    procedure act_CopyNumExecute(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function  GetAllParams(var params:string):boolean;
    procedure LoadCallLog;
    procedure ShowCallLog;

    procedure WMUserActivate(var msg:TMessage); message WM_USER_ACTIVATE_PREV;
  end;

var
  frm_Main: Tfrm_Main;
  LogFile: string;
  slCalls: TStringList;
  LastSort: integer;
  SortNormal: boolean;

implementation

{$R *.dfm}

function CustomSortProc(Item1, Item2: TListItem; ColNumber: integer): integer; stdcall;
begin
  result := CompareText(Item1.SubItems.Strings[ColNumber - 1], Item2.SubItems.Strings[ColNumber - 1]);

  if not SortNormal then
    result := -result;
end;

function GetNextLineItem(var s: string): string;
var
  i: integer;
begin
  i := Pos(DELIM, s);

  if i = 0 then begin
    result := Trim(s);
    s      := '';
   end
  else begin
    result := Trim(Copy(s, 1, i - 1));
    Delete(s, 1, i);
  end;
end;

procedure Tfrm_Main.act_CopyExecute(Sender: TObject);
var
  clip: TClipboard;
begin
  clip := TClipboard.Create;
  clip.SetTextBuf(PChar(lst_Calls.Selected.Caption + DELIM + lst_Calls.Selected.SubItems.DelimitedText));
  clip.Free;
end;

procedure Tfrm_Main.act_CopyNumExecute(Sender: TObject);
var
  clip: TClipboard;
begin
  clip := TClipboard.Create;
  clip.SetTextBuf(PChar(lst_Calls.Selected.SubItems.Strings[2]));
  clip.Free;
end;

procedure Tfrm_Main.act_GoogleExecute(Sender: TObject);
begin
  ShellExecute(Handle, nil, PChar('http://www.google.com/search?q=' + lst_Calls.Selected.SubItems.Strings[2]), nil, nil, SW_SHOWNORMAL);
end;

procedure Tfrm_Main.FormCreate(Sender: TObject);
var
  h: THandle;
begin
  // Check for previous instance
  CreateMutex(nil, true, TITLE);

  if GetLastError <> NO_ERROR then begin
    h := FindWindow(PChar(String(ClassName)), PChar(TITLE));

    if h > 0 then
      PostMessage(h, WM_USER_ACTIVATE_PREV, 0, 0);

    Application.Terminate;
    exit;
  end;

  Caption := TITLE;

  if not GetAllParams(LogFile) then begin
    Application.Terminate;
    exit;
  end;

  slCalls := TStringList.Create;

  SortNormal := true;
  LastSort   := 0;

  snap.SnapTo(spRight);
  snap.SnapTo(spBottom);
  snap.Active := true;

  Show;

  LoadCallLog;
end;

procedure Tfrm_Main.FormDestroy(Sender: TObject);
begin
  slCalls.Free;
end;

function Tfrm_Main.GetAllParams(var params:string):boolean;
var
  i: integer;
begin
  params := '';

  for i := 1 to ParamCount do
    params := params + ParamStr(i) + ' ';

  params := Trim(params);

  result := params <> '';
end;

procedure Tfrm_Main.LoadCallLog;
var
  tf: textfile;
  s, s2: string;
  i: integer;
begin
  Screen.Cursor := crHourglass;

  stat.SimpleText := LOADING_LOG;
  stat.Update;

  AssignFile(tf, LogFile);
  Reset(tf);

  slCalls.Clear;

  while not Eof(tf) do begin
    Readln(tf, s);

    if Copy(s, 1, Length(CALL_SIG)) = CALL_SIG then begin
      s := Trim(s);
      s := Trim(StringReplace(Copy(s, Length(CALL_SIG) + 1, Length(s) - Length(CALL_SIG)), '*', '', [rfReplaceAll]));
      i := Pos('-', s);

      s2 := Trim(Copy(s, 1, i - 1)) + DELIM + Trim(Copy(s, i + 1, Length(s) - i));

      Readln(tf, s);
      s := Trim(s);

      s2 := s2 + DELIM + Trim(Copy(s, Length(NAME_SIG) + 1, Length(s) - Length(NAME_SIG)));

      Readln(tf, s);
      s := Trim(s);

      s2 := s2 + DELIM + Trim(Copy(s, Length(NUMB_SIG) + 1, Length(s) - Length(NUMB_SIG)));

      slCalls.Add(s2);
    end;
  end;

  CloseFile(tf);

  ShowCallLog;
end;

procedure Tfrm_Main.ShowCallLog;
var
  li: TListItem;
  i: integer;

  procedure AddCall;
  var
    s: string;
  begin
    s := slCalls.Strings[i - 1];

    li := lst_Calls.Items.Add;
    li.Caption := GetNextLineItem(s);

    li.SubItems.Delimiter := DELIM;
    li.SubItems.StrictDelimiter := true;
    li.SubItems.DelimitedText := s;
  end;

begin
  Screen.Cursor := crHourglass;

  stat.SimpleText := LOADING_LOG;
  stat.Update;

  lst_Calls.Clear;
  lst_Calls.Update;
  lst_Calls.Items.BeginUpdate;

  if (LastSort = 0) and (not SortNormal) then
    for i := 1 to slCalls.Count do
      AddCall
  else
    for i := slCalls.Count downto 1 do
      AddCall;

  stat.SimpleText := IntToStr(slCalls.Count) + ' calls';

  lst_Calls.Items.EndUpdate;

  Screen.Cursor := crDefault;
end;

procedure Tfrm_Main.WMUserActivate(var msg: TMessage);
begin
  Application.Restore;
  Application.BringToFront;
  Application.ProcessMessages;

  LoadCallLog;

  msg.Result := 0;
end;

procedure Tfrm_Main.lst_CallsColumnClick(Sender: TObject; Column: TListColumn);
var
  NewSort: integer;
begin
  NewSort := Column.Index;

  // Date/Time sorted together
  if NewSort = 1 then
    NewSort := 0;

  // Toggle sort order
  if NewSort = LastSort then
    SortNormal := not SortNormal
  else
    SortNormal := true;

  LastSort := NewSort;

  if NewSort = 0 then
    ShowCallLog
  else
    lst_Calls.CustomSort(@CustomSortProc, NewSort);
end;

procedure Tfrm_Main.lst_CallsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  b: boolean;
begin
  b := Assigned(lst_Calls.Selected);

  act_CopyNum.Enabled := b;
  act_Copy.Enabled    := b;
  act_Google.Enabled  := b;
end;

end.
